import sys
import time
import logging
from cStringIO import StringIO

from compiler import UICompiler
import indenter
from PyQt4.uic.exceptions import *
from PyQt4.QtCore import PYQT_VERSION_STR as VERSION

_header = """# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '%s'
#
# Created: %s
#      by: PyQt4 UI code generator v%s
#
# WARNING! All changes made in this file will be lost!

import sys
"""

_democode = """
if __name__ == "__main__":
\tapp = QtGui.QApplication(sys.argv)
\t%(uiname)s = QtGui.%(baseclass)s()
\tui = %(uiclass)s()
\tui.setupUi(%(uiname)s)
\t%(uiname)s.show()
\tsys.exit(app.exec_())
"""


def compileUi(input_stream, output_stream):
    """compileUi(input_stream, output_stream) -> dict

    Generates Python code from a UI definition file
    
    input_stream is the stream that contains the UI definition
    The code will be written into output_stream

    >>> import cStringIO
    >>> compileUi(file('demo.ui'), cStringIO.StringIO)
    {'uiname'    : <name of toplevel form>,
     'uiclass'   : <name of the UI class, defaults to 'Ui_' + uiname>,
     'baseclass' : <base class of the toplevel form (QWidget, QDialog> }

    """
    ui_compiler = UICompiler()
    return ui_compiler.compileUi(input_stream, output_stream)


def generatePythonModule(input_filename, output_filename,
                         add_display_code = False, indent_width = 4):
    """generatePythonModule(input_filename, output_filename,
                            add_display_code, indent_width) -> int

    Creates a Python modules from a UI definition file.
    
    input_filename is the name of the UI file, or - for stdin
    output_filename is the name of the file to be generated, or - for stdout
    If add_display_code is true, test code for displaying the widget will
    be written. Default: False
    ident_width is the indentation width in spaces. Default: 4
    """
    indenter.indentwidth = indent_width
    if input_filename == "-":
        input_stream = sys.stdin
    else:
        input_stream = file(input_filename, "r")

    code_buffer = StringIO()
    code_buffer.write(_header % (input_filename, time.ctime(), VERSION))
    widget_info = compileUi(input_stream, code_buffer)
    if add_display_code:
        code_buffer.write(_democode.replace("\t", " " * indent_width) \
                          % widget_info)

    if output_filename == "-":
        sys.stdout.write(code_buffer.getvalue())
    else:
        file(output_filename, "w").write(code_buffer.getvalue())

    return 0

def compileUiToType(filename):
    """compileUiToType(filename) -> new class

    Compile a UI definition and return the generated class.
    filename is the name of the UI definition.
    """
    code_string = StringIO()
    widget_info = compileUi(file(filename, "r"), code_string)
    ui_globals = {}
    exec code_string.getvalue() in ui_globals
    return ui_globals[widget_info["uiclass"]]

